@@ -1,15 +1,17 @@
 ## functions plot stochastic character mapped trees
-## written by Liam Revell 2011-2017
+## written by Liam Revell 2011-2020
 
 plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 	pts=FALSE,node.numbers=FALSE,mar=NULL,add=FALSE,offset=NULL,direction="rightwards",
 	type="phylogram",setEnv=TRUE,part=1.0,xlim=NULL,ylim=NULL,nodes="intermediate",
-	tips=NULL,maxY=NULL,hold=TRUE,split.vertical=FALSE,lend=2,asp=NA){
+	tips=NULL,maxY=NULL,hold=TRUE,split.vertical=FALSE,lend=2,asp=NA,outline=FALSE,
+	plot=TRUE){
 	if(inherits(tree,"multiPhylo")){
 		par(ask=TRUE)
-		for(i in 1:length(tree)) plotSimmap(tree[[i]],colors=colors,fsize=fsize,ftype=ftype,
-			lwd=lwd,pts=pts,node.numbers=node.numbers,mar,add,offset,direction,type,
-			setEnv,part,xlim,ylim,nodes,tips,maxY,hold,split.vertical,lend)
+		for(i in 1:length(tree)) plotSimmap(tree[[i]],colors=colors,fsize=fsize,
+			ftype=ftype,lwd=lwd,pts=pts,node.numbers=node.numbers,mar,add,offset,
+			direction,type,setEnv,part,xlim,ylim,nodes,tips,maxY,hold,split.vertical,
+			lend,asp,outline,plot)
 	} else {
 		# check tree
 		if(!inherits(tree,"phylo")) stop("tree should be object of class \"phylo\"")
@@ -33,14 +35,59 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 		if(is.null(mar)) mar=rep(0.1,4)
 		if(hold) null<-dev.hold()
 		if(type=="phylogram"){
-			if(direction%in%c("upwards","downwards"))
-				updownPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,add,offset,
-					direction,setEnv,xlim,ylim,nodes,tips,split.vertical,lend,asp)
-			else plotPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,add,offset,
-					direction,setEnv,xlim,ylim,nodes,tips,split.vertical,lend,asp)
+			if(direction%in%c("upwards","downwards")){
+				if(outline){
+					fg<-par()$fg
+					par(fg="transparent")
+					black<-colors
+					black[]<-"black"
+					updownPhylogram(tree,colors=black,fsize,ftype,lwd=lwd+2,pts,
+						node.numbers,mar,add,offset,direction,setEnv,xlim,ylim,nodes,
+						tips,split.vertical,lend,asp,plot)
+					par(fg=fg)
+				}
+				updownPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
+				add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
+				tips,split.vertical,lend,asp,plot)
+			} else {
+				if(outline){
+					fg<-par()$fg
+					par(fg="transparent")
+					black<-colors
+					black[]<-"black"
+					plotPhylogram(tree,colors=black,fsize,ftype,lwd=lwd+2,pts,
+						node.numbers,mar,add,offset,direction,setEnv,xlim,ylim,nodes,
+						tips,split.vertical,lend,asp,plot)
+					par(fg=fg)
+				}
+				plotPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
+				add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
+				tips,split.vertical,lend,asp,plot)
+			}
 		} else if(type=="fan"){
-			plotFan(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips,
-				maxY,lend)
+			if(outline){
+				fg<-par()$fg
+				par(fg="transparent")
+				black<-colors
+				black[]<-"black"
+				plotFan(tree,colors=black,fsize,ftype,lwd=lwd+2,mar,add,part,setEnv,
+					xlim,ylim,tips,maxY,lend,plot,offset)
+				par(fg=fg)
+			}
+			plotFan(tree,colors,fsize,ftype,lwd,mar,add=if(outline) TRUE else add,part,
+				setEnv,xlim,ylim,tips,maxY,lend,plot,offset)
+		} else if(type=="cladogram"){
+			if(outline){
+				fg<-par()$fg
+				par(fg="transparent")
+				black<-colors
+				black[]<-"black"
+				plotCladogram(tree,colors=black,fsize,ftype,lwd=lwd+2,mar,add,offset,
+					direction,xlim,ylim,nodes,tips,lend,asp,plot)
+				par(fg=fg)
+			}
+			plotCladogram(tree,colors,fsize,ftype,lwd,mar,add=if(outline) TRUE else add,
+				offset,direction,xlim,ylim,nodes,tips,lend,asp,plot)
 		}
 		if(hold) null<-dev.flush()
 	}
@@ -50,7 +97,7 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 ## written by Liam J. Revell 2011-2017
 updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	add,offset,direction,setEnv,xlim,ylim,placement,tips,split.vertical,lend,
-	asp){
+	asp,plot){
 	if(split.vertical&&!setEnv){
 		cat("split.vertical requires setEnv=TRUE. Setting split.vertical to FALSE.\n")
 		spit.vertical<-FALSE
@@ -115,57 +162,59 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	if(direction=="downwards") H<-max(H)-H
 	plot.window(xlim=xlim,ylim=ylim,asp=asp)
 	####
-	if(!split.vertical){
-		for(i in 1:m) lines(Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],
-			H[which(cw$edge[,1]==nodes[i]),1],
-			col=colors[names(cw$maps[[match(nodes[i],
-			cw$edge[,1])]])[1]],lwd=lwd)
-	}
-	for(i in 1:nrow(cw$edge)){
-		x<-H[i,1]
- 		for(j in 1:length(cw$maps[[i]])){
-			if(direction=="downwards")
-				lines(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x-cw$maps[[i]][j]),
-					col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
-			else lines(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x+cw$maps[[i]][j]),
-					col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
-			if(pts) points(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x+cw$maps[[i]][j]),
-				pch=20,lwd=(lwd-1))
-			x<-x+if(direction=="downwards") -cw$maps[[i]][j] else cw$maps[[i]][j]
-			j<-j+1
+	if(plot){
+		if(!split.vertical){
+			for(i in 1:m) lines(Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],
+				H[which(cw$edge[,1]==nodes[i]),1],
+				col=colors[names(cw$maps[[match(nodes[i],
+				cw$edge[,1])]])[1]],lwd=lwd)
 		}
-	}
-	if(node.numbers){
-		symbols(mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
-			if(direction=="downwards") max(H) else 0,
-			rectangles=matrix(c(1.2*fsize*strwidth(as.character(Ntip(cw)+1)),
-			1.4*fsize*strheight(as.character(Ntip(cw)+1))),1,2),inches=FALSE,
-			bg="white",add=TRUE)
-		text(mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
-			if(direction=="downwards") max(H) else 0,Ntip(cw)+1,
-			cex=fsize)
 		for(i in 1:nrow(cw$edge)){
-			x<-H[i,2]
-			if(cw$edge[i,2]>Ntip(cw)){
-				symbols(Y[cw$edge[i,2]],x,
-					rectangles=matrix(c(1.2*fsize*strwidth(as.character(cw$edge[i,2])),
-					1.4*fsize*strheight(as.character(cw$edge[i,2]))),1,2),inches=FALSE,
-					bg="white",add=TRUE)
-				text(Y[cw$edge[i,2]],x,cw$edge[i,2],cex=fsize)
+			x<-H[i,1]
+			for(j in 1:length(cw$maps[[i]])){
+				if(direction=="downwards")
+					lines(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x-cw$maps[[i]][j]),
+						col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+				else lines(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x+cw$maps[[i]][j]),
+						col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+				if(pts) points(c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),c(x,x+cw$maps[[i]][j]),
+					pch=20,lwd=(lwd-1))
+				x<-x+if(direction=="downwards") -cw$maps[[i]][j] else cw$maps[[i]][j]
+				j<-j+1
 			}
 		}
-	}
-	if(direction=="downwards") pos<-if(par()$usr[3]>par()$usr[4]) 2 else 4
-	if(direction=="upwards") pos<-if(par()$usr[3]>par()$usr[4]) 2 else 4
-	for(i in 1:n){
-		shift<-offset*fsize*strwidth("W")*(diff(par()$usr[3:4])/diff(par()$usr[1:2]))
-		if((direction=="downwards"&&diff(par()$usr[3:4])>0) ||
-			(direction=="upwards"&&diff(par()$usr[3:4])<0)) shift<--shift
-		if(ftype){
-			text(labels=cw$tip.label[i],Y[i],
-				H[which(cw$edge[,2]==i),2]+shift,
-				pos=pos,offset=0,cex=fsize,font=ftype,
-				srt=if(direction=="downwards") 270 else 90)
+		if(node.numbers){
+			symbols(mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
+				if(direction=="downwards") max(H) else 0,
+				rectangles=matrix(c(1.2*fsize*strwidth(as.character(Ntip(cw)+1)),
+				1.4*fsize*strheight(as.character(Ntip(cw)+1))),1,2),inches=FALSE,
+				bg="white",add=TRUE)
+			text(mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
+				if(direction=="downwards") max(H) else 0,Ntip(cw)+1,
+				cex=fsize)
+			for(i in 1:nrow(cw$edge)){
+				x<-H[i,2]
+				if(cw$edge[i,2]>Ntip(cw)){
+					symbols(Y[cw$edge[i,2]],x,
+						rectangles=matrix(c(1.2*fsize*strwidth(as.character(cw$edge[i,2])),
+						1.4*fsize*strheight(as.character(cw$edge[i,2]))),1,2),inches=FALSE,
+						bg="white",add=TRUE)
+					text(Y[cw$edge[i,2]],x,cw$edge[i,2],cex=fsize)
+				}
+			}
+		}
+		if(direction=="downwards") pos<-if(par()$usr[3]>par()$usr[4]) 2 else 4
+		if(direction=="upwards") pos<-if(par()$usr[3]>par()$usr[4]) 2 else 4
+		for(i in 1:n){
+			shift<-offset*fsize*strwidth("W")*(diff(par()$usr[3:4])/diff(par()$usr[1:2]))
+			if((direction=="downwards"&&diff(par()$usr[3:4])>0) ||
+				(direction=="upwards"&&diff(par()$usr[3:4])<0)) shift<--shift
+			if(ftype){
+				text(labels=cw$tip.label[i],Y[i],
+					H[which(cw$edge[,2]==i),2]+shift,
+					pos=pos,offset=0,cex=fsize,font=ftype,
+					srt=if(direction=="downwards") 270 else 90)
+			}
 		}
 	}
 	if(setEnv){
@@ -178,14 +227,14 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 			function(x,y,z) y[match(x,z)],y=H,z=cw$edge))
 		assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	}
-	if(split.vertical) splitEdgeColor(cw,colors,lwd)
+	if(plot) if(split.vertical) splitEdgeColor(cw,colors,lwd)
 }
 
 # function to plot simmap tree in type "phylogram"
-# written by Liam J. Revell 2011-2015
+# written by Liam J. Revell 2011-2017
 plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	add,offset,direction,setEnv,xlim,ylim,placement,tips,split.vertical,lend,
-	asp){
+	asp,plot){
 	if(split.vertical&&!setEnv){
 		cat("split.vertical requires setEnv=TRUE. Setting split.vertical to FALSE.\n")
 		spit.vertical<-FALSE
@@ -249,50 +298,52 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	if(is.null(ylim)) ylim=range(Y)
 	if(direction=="leftwards") H<-max(H)-H
 	plot.window(xlim=xlim,ylim=ylim,asp=asp)
-	####
-	if(!split.vertical){
-		for(i in 1:m) lines(H[which(cw$edge[,1]==nodes[i]),1],
-			Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],col=colors[names(cw$maps[[match(nodes[i],
-			cw$edge[,1])]])[1]],lwd=lwd)
-	}
-	for(i in 1:nrow(cw$edge)){
-		x<-H[i,1]
- 		for(j in 1:length(cw$maps[[i]])){
-			if(direction=="leftwards")
-				lines(c(x,x-cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
-					col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
-			else lines(c(x,x+cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
-					col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
-			if(pts) points(c(x,x+cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
-				pch=20,lwd=(lwd-1))
-			x<-x+if(direction=="leftwards") -cw$maps[[i]][j] else cw$maps[[i]][j]
-			j<-j+1
+	if(plot){
+		####
+		if(!split.vertical){
+			for(i in 1:m) lines(H[which(cw$edge[,1]==nodes[i]),1],
+				Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],col=colors[names(cw$maps[[match(nodes[i],
+				cw$edge[,1])]])[1]],lwd=lwd)
 		}
-	}
-	if(node.numbers){
-		symbols(if(direction=="leftwards") max(H) else 0,
-			mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
-			rectangles=matrix(c(1.2*fsize*strwidth(as.character(Ntip(cw)+1)),
-			1.4*fsize*strheight(as.character(Ntip(cw)+1))),1,2),inches=FALSE,
-			bg="white",add=TRUE)
-		text(if(direction=="leftwards") max(H) else 0,
-			mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),Ntip(cw)+1,
-			cex=fsize)
 		for(i in 1:nrow(cw$edge)){
-			x<-H[i,2]
-			if(cw$edge[i,2]>Ntip(cw)){
-				symbols(x,Y[cw$edge[i,2]],
-					rectangles=matrix(c(1.2*fsize*strwidth(as.character(cw$edge[i,2])),
-					1.4*fsize*strheight(as.character(cw$edge[i,2]))),1,2),inches=FALSE,
-					bg="white",add=TRUE)
-				text(x,Y[cw$edge[i,2]],cw$edge[i,2],cex=fsize)
+			x<-H[i,1]
+			for(j in 1:length(cw$maps[[i]])){
+				if(direction=="leftwards")
+					lines(c(x,x-cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
+						col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+				else lines(c(x,x+cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
+						col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+				if(pts) points(c(x,x+cw$maps[[i]][j]),c(Y[cw$edge[i,2]],Y[cw$edge[i,2]]),
+					pch=20,lwd=(lwd-1))
+				x<-x+if(direction=="leftwards") -cw$maps[[i]][j] else cw$maps[[i]][j]
+				j<-j+1
 			}
 		}
+		if(node.numbers){
+			symbols(if(direction=="leftwards") max(H) else 0,
+				mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),
+				rectangles=matrix(c(1.2*fsize*strwidth(as.character(Ntip(cw)+1)),
+				1.4*fsize*strheight(as.character(Ntip(cw)+1))),1,2),inches=FALSE,
+				bg="white",add=TRUE)
+			text(if(direction=="leftwards") max(H) else 0,
+				mean(Y[cw$edge[cw$edge[,1]==(Ntip(cw)+1),2]]),Ntip(cw)+1,
+				cex=fsize)
+			for(i in 1:nrow(cw$edge)){
+				x<-H[i,2]
+				if(cw$edge[i,2]>Ntip(cw)){
+					symbols(x,Y[cw$edge[i,2]],
+						rectangles=matrix(c(1.2*fsize*strwidth(as.character(cw$edge[i,2])),
+						1.4*fsize*strheight(as.character(cw$edge[i,2]))),1,2),inches=FALSE,
+						bg="white",add=TRUE)
+					text(x,Y[cw$edge[i,2]],cw$edge[i,2],cex=fsize)
+				}
+			}
+		}
+		if(direction=="leftwards") pos<-if(par()$usr[1]>par()$usr[2]) 4 else 2
+		if(direction=="rightwards") pos<-if(par()$usr[1]>par()$usr[2]) 2 else 4
+		for(i in 1:n) if(ftype) text(H[which(cw$edge[,2]==i),2],Y[i],cw$tip.label[i],pos=pos,
+			offset=offset,cex=fsize,font=ftype)
 	}
-	if(direction=="leftwards") pos<-if(par()$usr[1]>par()$usr[2]) 4 else 2
-	if(direction=="rightwards") pos<-if(par()$usr[1]>par()$usr[2]) 2 else 4
-	for(i in 1:n) if(ftype) text(H[which(cw$edge[,2]==i),2],Y[i],cw$tip.label[i],pos=pos,
-		offset=offset,cex=fsize,font=ftype)
 	if(setEnv){
 		PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
 			show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
@@ -303,12 +354,13 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 			function(x,y,z) y[match(x,z)],y=H,z=cw$edge),yy=Y[,1])
 		assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	}
-	if(split.vertical) splitEdgeColor(cw,colors,lwd)
+	if(plot) if(split.vertical) splitEdgeColor(cw,colors,lwd)
 }
 
 # function to plot simmap tree in type "fan"
-# written by Liam J. Revell 2013-2015
-plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips,maxY,lend){
+# written by Liam J. Revell 2013-2017
+plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips,maxY,lend,plot,offset=0){
+	if(!plot) cat("plot=FALSE option is not permitted for type=\"fan\". Tree will be plotted.\n")
 	# reorder
 	cw<-reorder(tree)
 	pw<-reorder(tree,"pruningwise")
@@ -335,10 +387,10 @@ plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips
 	# optimize x & y limits
 	par(mar=mar)
 	offsetFudge<-1.37 # empirically determined
-	offset<-0
+	OFFSET<-0
 	pp<-par("pin")[1]
  	sw<-fsize*(max(strwidth(cw$tip.label,units="inches")))+
-		offsetFudge*offset*fsize*strwidth("W",units="inches") 
+		offsetFudge*OFFSET*fsize*strwidth("W",units="inches") 
 	alp<-optimize(function(a,H,sw,pp) (2*a*1.04*max(H)+2*sw-pp)^2,H=R,sw=sw,pp=pp,
 		interval=c(0,1e6))$minimum
 	if(part<=0.25) x.lim<-y.lim<-c(0,max(R)+sw/alp)
@@ -388,7 +440,8 @@ plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips
 		ii<-which(cw$edge[,2]==i)
 		aa<-Y[ii,2]/(2*pi)*360
 		adj<-if(aa>90&&aa<270) c(1,0.25) else c(0,0.25)
-		tt<-if(aa>90&&aa<270) paste(cw$tip.label[i]," ",sep="") else paste(" ",
+		tt<-if(aa>90&&aa<270) paste(cw$tip.label[i],paste(rep(" ",offset),
+			collapse=""),sep="") else paste(paste(rep(" ",offset),collapse=""),
 			cw$tip.label[i],sep="")
 		aa<-if(aa>90&&aa<270) 180+aa else aa
 		if(ftype) text(x[ii,2],y[ii,2],tt,srt=aa,adj=adj,cex=fsize,font=ftype)
@@ -407,8 +460,111 @@ plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips
 	}
 }
 
+## internal function for slanted cladogram
+## written by Liam J. Revell 2017
+plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
+	add=FALSE,offset=NULL,direction="rightwards",xlim=NULL,ylim=NULL,
+	nodes="intermediate",tips=NULL,lend=2,asp=NA,plot=TRUE){
+	placement<-nodes
+	# set offset fudge (empirically determined)
+	offsetFudge<-1.37
+	# reorder
+	cw<-reorderSimmap(tree)
+	pw<-reorderSimmap(tree,"postorder")
+	# count nodes and tips
+	n<-Ntip(cw)
+ 	m<-cw$Nnode
+	# Y coordinates for nodes
+	Y<-matrix(NA,m+n,1)
+	# first, assign y coordinates to all the tip nodes
+	if(is.null(tips)) Y[cw$edge[cw$edge[,2]<=n,2]]<-1:n
+	else Y[cw$edge[cw$edge[,2]<=n,2]]<-if(is.null(names(tips))) 
+		tips[sapply(1:Ntip(cw),function(x,y) which(y==x),y=cw$edge[cw$edge[,2]<=n,2])]
+		else tips[gsub(" ","_",cw$tip.label)]
+	# get Y coordinates of the nodes
+	nodes<-unique(pw$edge[,1])
+	for(i in 1:m){
+		if(placement=="intermediate"){ 
+			desc<-pw$edge[which(pw$edge[,1]==nodes[i]),2]
+			Y[nodes[i]]<-(min(Y[desc])+max(Y[desc]))/2
+		} else if(placement=="centered"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			Y[nodes[i]]<-(min(Y[desc])+max(Y[desc]))/2
+		} else if(placement=="weighted"){
+			desc<-pw$edge[which(pw$edge[,1]==nodes[i]),2]
+			n1<-desc[which(Y[desc]==min(Y[desc]))]
+			n2<-desc[which(Y[desc]==max(Y[desc]))]
+			v1<-pw$edge.length[which(pw$edge[,2]==n1)]
+			v2<-pw$edge.length[which(pw$edge[,2]==n2)]
+			Y[nodes[i]]<-((1/v1)*Y[n1]+(1/v2)*Y[n2])/(1/v1+1/v2)
+		} else if(placement=="inner"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(abs(Y[desc]-median(Y[1:Ntip(pw)]))==min(abs(Y[desc]-
+				median(Y[1:Ntip(pw)]))))
+			if(length(mm>1)) mm<-mm[which(Y[desc][mm]==min(Y[desc][mm]))]
+			Y[nodes[i]]<-Y[desc][mm]
+		}
+	}
+	# compute node heights
+	H<-nodeHeights(cw)
+	# open plot
+	par(mar=mar)
+	if(is.null(offset)) offset<-0.2*lwd/3+0.2/3
+	if(!add) plot.new()
+	###
+	if(is.null(xlim)){
+		pp<-par("pin")[1]
+		sw<-fsize*(max(strwidth(cw$tip.label,units="inches")))+
+			offsetFudge*fsize*strwidth("W",units="inches")
+		alp<-optimize(function(a,H,sw,pp) (a*1.04*max(H)+sw-pp)^2,H=H,sw=sw,pp=pp,
+			interval=c(0,1e6))$minimum
+		xlim<-if(direction=="leftwards") c(min(H)-sw/alp,max(H)) else c(min(H),max(H)+sw/alp)
+	}
+	if(is.null(ylim)) ylim=range(Y)
+	if(direction=="leftwards") H<-max(H)-H
+	plot.window(xlim=xlim,ylim=ylim,asp=asp)
+	if(plot){
+		####
+		for(i in 1:nrow(cw$edge)){
+			x<-H[i,1]
+			y<-Y[cw$edge[i,1]]
+			m<-(Y[cw$edge[i,2]]-Y[cw$edge[i,1]])/(H[i,2]-H[i,1])
+			if(is.finite(m)){
+				for(j in 1:length(cw$maps[[i]])){
+					if(direction=="leftwards")
+						lines(c(x,x-cw$maps[[i]][j]),c(y,y-cw$maps[[i]][j]*m),
+							col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+					else lines(c(x,x+cw$maps[[i]][j]),c(y,y+cw$maps[[i]][j]*m),
+						col=colors[names(cw$maps[[i]])[j]],lwd=lwd,lend=lend)
+					x<-x+if(direction=="leftwards") -cw$maps[[i]][j] else cw$maps[[i]][j]
+					y<-y+if(direction=="leftwards") -m*cw$maps[[i]][j] else m*cw$maps[[i]][j]
+					j<-j+1
+				}
+			} else {
+				lines(rep(x,2),Y[cw$edge[i,]],col=colors[names(cw$maps[[i]])[1]],lwd=lwd,
+					lend=lend)
+			}
+		}
+		if(direction=="leftwards") pos<-if(par()$usr[1]>par()$usr[2]) 4 else 2
+		if(direction=="rightwards") pos<-if(par()$usr[1]>par()$usr[2]) 2 else 4
+		for(i in 1:n) if(ftype) text(H[which(cw$edge[,2]==i),2],Y[i],cw$tip.label[i],pos=pos,
+			offset=offset,cex=fsize,font=ftype)
+	}
+	PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
+		show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
+		font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=offset,
+		x.lim=xlim,y.lim=ylim,
+		direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
+		edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
+		function(x,y,z) y[match(x,z)],y=H,z=cw$edge),yy=Y[,1])
+	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
+}
+
+
 ## adds legend to an open stochastic map style plot
-## written by Liam J. Revell 2013, 2016
+## written by Liam J. Revell 2013, 2016, 2017
 add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
 	if(hasArg(shape)) shape<-list(...)$shape
 	else shape<-"square"
@@ -428,13 +584,14 @@ add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
 	if(is.null(leg)) leg<-names(colors)
 	h<-fsize*strheight(LETTERS[1])
 	w<-par()$mfcol[2]*h*abs(diff(par()$usr[1:2])/diff(par()$usr[3:4]))
+	flipped<-par()$usr[1]>par()$usr[2]
 	if(vertical){
 		y<-y-0:(length(leg)-1)*1.5*h
-		x<-rep(x+w/2,length(y))		
-		text(x+w,y,leg,pos=4,cex=fsize/par()$cex)
+		x<-rep(x+w/2,length(y))
+		text(x + if(flipped) -w else w,y,leg,pos=4,cex=fsize/par()$cex)
 	} else {
-		sp<-fsize*max(strwidth(leg))
-		x<-x-w/2+0:(length(leg)-1)*1.5*(sp+w)
+		sp<-abs(fsize*max(strwidth(leg)))
+		x<-x + if(flipped) w/2-0:(length(leg)-1)*1.5*(sp+w) else -w/2+0:(length(leg)-1)*1.5*(sp+w)
 		y<-rep(y+w/2,length(x))
 		text(x,y,leg,pos=4,cex=fsize/par()$cex)
 	}
@@ -445,7 +602,7 @@ add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
 }
 
 # function plots a tree; in the new version this is just a wrapper for plotSimmap
-# written by Liam Revell 2012-2015
+# written by Liam Revell 2012-2017
 plotTree<-function(tree,...){
 	if(hasArg(color)) color<-list(...)$color
 	else color<-NULL
@@ -489,13 +646,15 @@ plotTree<-function(tree,...){
 	else lend<-2
 	if(hasArg(asp)) asp<-list(...)$asp
 	else asp<-NA
+	if(hasArg(plot)) plot<-list(...)$plot
+	else plot<-TRUE
 	if(inherits(tree,"multiPhylo")){
 		par(ask=TRUE)
 		if(!is.null(color)) names(color)<-"1"
 		for(i in 1:length(tree)) plotTree(tree[[i]],color=color,fsize=fsize,ftype=ftype,
 			lwd=lwd,pts=pts,node.numbers=node.numbers,mar=mar,add=add,offset=offset,
 			direction=direction,type=type,setEnv=setEnv,part=part,xlim=xlim,ylim=ylim,
-			nodes=nodes,tips=tips,maxY=maxY,hold=hold,lend=lend,asp=asp)
+			nodes=nodes,tips=tips,maxY=maxY,hold=hold,lend=lend,asp=asp,plot=plot)
 	} else {
 		if(is.null(tree$edge.length)) tree<-compute.brlen(tree)
 		tree$maps<-as.list(tree$edge.length)
@@ -504,7 +663,7 @@ plotTree<-function(tree,...){
 		plotSimmap(tree,colors=color,fsize=fsize,ftype=ftype,lwd=lwd,pts=pts,
 			node.numbers=node.numbers,mar=mar,add=add,offset=offset,direction=direction,
 			type=type,setEnv=setEnv,part=part,xlim=xlim,ylim=ylim,nodes=nodes,tips=tips,maxY=maxY,
-			hold=hold,lend=lend,asp=asp)
+			hold=hold,lend=lend,asp=asp,plot=plot)
 	}
 }
 
@@ -535,3 +694,4 @@ splitEdgeColor<-function(tree,colors,lwd=2){
 	}
 }
 
+
